@@ -2,8 +2,10 @@ package Yote::AppRoot;
 
 use strict;
 
+use Yote::Account;
 use Yote::Obj;
-use Yote::AccountRoot;
+use Yote::SystemObj;
+use Yote::YoteRoot;
 
 use Crypt::Passwd;
 use Email::Valid;
@@ -14,7 +16,7 @@ use vars qw($VERSION);
 
 $VERSION = '0.01';
 
-use base 'Yote::Obj';
+use base 'Yote::SystemObj';
 
 sub _on_load {
     my $self = shift;
@@ -24,19 +26,19 @@ sub _on_load {
 #
 # Returns the account root attached to this AppRoot for the given account.
 #
-sub account_root {
+sub _account_root {
     my( $self, $acct ) = @_;
 
     my $acct_roots = $self->get_account_roots({});
     my $root = $acct_roots->{$acct->{ID}};
     unless( $root ) {
-        $root = new Yote::AccountRoot;
+        $root = new Yote::SystemObj;
         $root->set_account( $acct );
         $acct_roots->{$acct->{ID}} = $root;
     }
     return $root;
 
-} #account_root
+} #_account_root
 
 #
 # Process_command is only called on the master root, 
@@ -54,10 +56,8 @@ sub account_root {
 # either c or i must be given
 sub _process_command {
     my( $root, $cmd ) = @_;
-
     my $command = $cmd->{c};
-
-    my $data = _translate_data( $cmd->{d} );
+    my $data = _translate_data( $cmd->{data} );
 
     #
     # this will not process private (beginning with _) commands,
@@ -71,6 +71,9 @@ sub _process_command {
     elsif( $command eq 'login' ) {
         return _login( $data, $cmd->{oi} );
     }
+    elsif( $command eq 'verify_token' ) {
+        return { r => _fetch_root()->_obj_to_response(_valid_token( $data->{t}, $cmd->{oi} ) ) };
+    }
     elsif( $command eq 'remove_account' ) {
         return $root->_remove_account( $data, $acct );
     }
@@ -80,18 +83,39 @@ sub _process_command {
     elsif( $command eq 'reset_password' ) {
         return $root->_reset_password( $data );
     }
+    elsif( $command eq 'check_cron' ) {
+        if( $cmd->{cron_id} == $Yote::WebAppServer::cron_id  ) {
+            my $cron_pkg = $root->get_cron_package('Yote::Cron');
+            my $app = $root->get_apps()->{$cron_pkg};
+            unless( $app ) {
+                eval( "use $cron_pkg" );
+                if( $@ =~ /Can.t locate/ ) {
+                    print STDERR Data::Dumper->Dump(["No Cron Found"]);
+                    return { err => "App '$cron_pkg' not found" };
+                }
+                my $apps = $root->get_apps();
+                $app = $cron_pkg->new;
+                $apps->{$cron_pkg} = $app;		
+            }
+            $app->check_cron();
+        }
+    }
+    elsif( $command eq 'fetch_root' ) {
+	my $r = _fetch_root();
+        return { r => $r->_obj_to_response( $r ) };
+    }
     else {
-        my $appstr = $command eq 'fetch_root' && ref($data) ? $data->{app} : $cmd->{a};
+        my $appstr = $cmd->{a};
         my $app;
         if( $appstr ) {
             
-            $app = $root->get_apps({})->{$appstr};
+            $app = $root->get_apps()->{$appstr};
 
             #generate the app if not present.
             unless( $app ) {
                 eval( "use $appstr" );
                 if( $@ =~ /Can.t locate/ ) {
-                    return { err => "App '$a' not found" };
+                    return { err => "App '$appstr' not found" };
                 }
                 my $apps = $root->get_apps();
                 $app = $appstr->new;
@@ -102,24 +126,24 @@ sub _process_command {
             $app = $root;
         }            
 
-        if( $command eq 'fetch_root' ) {
-            return _fetch( $app, { id => $app->{ID} }, $acct );
+        if( $command eq 'fetch_app' ) {
+            return  _fetch( $app, $app->{ID}, $acct );
         }
         elsif( $command eq 'fetch' ) {
-            return _fetch( $app, { id => $data->{id} }, $acct );
+            return _fetch( $app, $cmd->{id}, $acct );
         }
         elsif( $command eq 'multi_fetch' ) {
             return _multi_fetch( $app, { ids => $data->{ids} }, $acct );
         }
         elsif( $command eq 'update' ) {
-            my $obj = Yote::ObjProvider::fetch( $data->{id} );
-            if( $obj && $app->_stow_permitted( $data->{d} ) ) {
+            my $obj = Yote::ObjProvider::fetch( $cmd->{id} );
+            if( $obj && $app->_stow_permitted( $data ) ) {
                 if( ref( $obj ) eq 'ARRAY' ) {
-                    @{$obj} = values %{$data->{d}};
+                    @{$obj} = values %{$data};
                 } elsif( ref( $obj ) eq 'HASH' ) {
-                    %{$obj} = map { $_ => $data->{d}{$_} } keys %{$data->{d}};
+                    %{$obj} = map { $_ => $data->{$_} } keys %{$data};
                 } else {
-                    $obj->absorb( $data->{d} );
+                    $obj->_absorb( $data );
                 }
                 return { msg => "updated" };
             }
@@ -127,10 +151,11 @@ sub _process_command {
         }
 
         my $obj = Yote::ObjProvider::fetch( $cmd->{id} ) || $app;
+
         if( $app->_allows( $command, $data, $acct, $obj ) && $obj->can( $command ) ) {
             my %before = map { $_ => 1 } (Yote::ObjProvider::dirty_ids());
             my $resp = $app->_obj_to_response( $obj->$command( $data,
-                                                               $app->account_root( $acct ),
+                                                               $app->_account_root( $acct ),
                                                                $acct ), 1 );
             my @dirty_delta = grep { ! $before{$_} } (Yote::ObjProvider::dirty_ids());
             return { r => $resp, d => \@dirty_delta };
@@ -139,12 +164,6 @@ sub _process_command {
     }
 } #_process_command
 
-# this makes the account root visible to javascript, which will have easier access to its methods.
-sub fetch_account_root {
-    my( $self, $data, $acct_root, $acct ) = @_;
-    return  { r => $acct_root };
-}
-
 sub _translate_data {
     my $val = shift;
 
@@ -161,15 +180,15 @@ sub _translate_data {
 #
 sub _allows { 
     my( $app, $command, $data, $acct, $obj ) = @_;
-    return 0 if index( $command, '_' ) == 0 || $command eq 'absorb';
+    return 0 if index( $command, '_' ) == 0;
     return 1;
 }
 
 #
 # Fetch master root singleton object.
 #
-sub fetch_root {
-    return Yote::ObjProvider::fetch( 1 ) || new Yote::AppRoot();
+sub _fetch_root {
+    return Yote::ObjProvider::fetch( 1 ) || new Yote::YoteRoot();
 }
 
 sub _valid_token {
@@ -187,7 +206,7 @@ sub _recover_password {
     my $email = $args->{e};
     my $from_url = $args->{u};
     my $to_reset = $args->{t};
-    my $acct = $root->get_emails({})->{$email};
+    my $acct = Yote::ObjProvider::xpath( "/emails/$email" );
     if( $acct ) {
         my $now = time();
         if( $now - $acct->get_last_recovery_time() > (60*15) ) { #need to wait 15 mins
@@ -244,7 +263,7 @@ sub _reset_password {
 
 sub _remove_account {
     my( $root, $args, $acct ) = @_;
-    if( $acct && _encrypt_pass($args->{p}, $acct) eq $acct->get_password() && $args->{h} eq $acct->get_handle() && $args->{e} eq $acct->get_email() ) {
+    if( $acct && _encrypt_pass($args->{p}, $acct) eq $acct->get_password() && $args->{h} eq $acct->get_handle() && $args->{e} eq $acct->get_email() && ! $acct->get_is_first_account() ) {
         delete $root->get_handles()->{$args->{h}};
         delete $root->get_emails()->{$args->{e}};
         return { r => "deleted account" };
@@ -259,7 +278,7 @@ sub _create_account {
     # validate account args. Needs handle (,email at some point)
     #
     my( $handle, $email, $password ) = ( $args->{h}, $args->{e}, $args->{p} );
-    if( $handle ) {# && $email ) {
+    if( $handle ) {
         if( Yote::ObjProvider::xpath("/handles/$handle") ) {
             return { err => "handle already taken" };
         }
@@ -274,7 +293,7 @@ sub _create_account {
         unless( $password ) {
             return { err => "password required" };
         }
-        my $newacct = new Yote::Obj();
+        my $newacct = new Yote::Account();
 
         #
         # check to see how many accounts there are. If there are none,
@@ -282,6 +301,9 @@ sub _create_account {
         #
         if( Yote::ObjProvider::xpath_count( "/handles" ) == 0 ) {
             $newacct->set_is_root( 1 );
+            $newacct->set_is_first_account( 1 );
+        } else {
+            $newacct->set_is_root( 0 );
         }
         $newacct->set_handle( $handle );
         $newacct->set_email( $email );
@@ -291,9 +313,9 @@ sub _create_account {
 
         $newacct->set_password( _encrypt_pass($password, $newacct) );
 
-        my $accts = $root->get_handles({});
+        my $accts = $root->get_handles();
         $accts->{ $handle } = $newacct;
-        my $emails = $root->get_emails({});
+        my $emails = $root->get_emails();
         $emails->{ $email } = $newacct;
         &Yote::ObjProvider::stow_all();
         return { r => "created account", a => $root->_obj_to_response( $newacct ), t => _create_token( $newacct, $ip ) };
@@ -316,9 +338,12 @@ sub _create_token {
 sub _login {
     my( $data, $ip ) = @_;
     if( $data->{h} ) {
-        my $root = fetch_root();
+        my $root = _fetch_root();
         my $acct = Yote::ObjProvider::xpath("/handles/$data->{h}");
         if( $acct && ($acct->get_password() eq _encrypt_pass( $data->{p}, $acct) ) ) {
+            if( $data->{rr} && ! $acct->get_is_root() ) {
+                return { err => "need a root login" };
+            }
             return { r => "logged in", a => $root->_obj_to_response( $acct ), t => _create_token( $acct, $ip ) };
         }
     }
@@ -335,18 +360,19 @@ sub _encrypt_pass {
 # Takes two args : object to be fetched and data of request.
 #
 sub _fetch_permitted {
-    my( $obj, $data ) = @_;
+    my( $obj, $id ) = @_;
     return 1;
 }
 
 
 #
-# Returns if the stow is allowed to proceed. Meant to override. Default is true.
+# Returns if the stow is allowed to proceed. Meant to override. 
+#     Default is true for anything not a system object (like app or account)
 # Takes two args : object to be stowed and data of request.
 #
 sub _stow_permitted {
     my( $obj, $data ) = @_;
-    return 1;
+    return $obj && $obj->isa( "Yote::SystemObj" );
 }
 
 #
@@ -355,17 +381,17 @@ sub _stow_permitted {
 #   d - key value data, where value can be a referece (is a number) or a scalar (is prepended with 'v' )
 #
 sub _fetch {
-    my( $app, $data, $acct ) = @_;
-    if( $data->{id} ) {
-        my $obj = Yote::ObjProvider::fetch( $data->{id} );
+    my( $app, $id, $acct ) = @_;
+    if( $id ) {
+        my $obj = Yote::ObjProvider::fetch( $id );
         if( $obj &&
             Yote::ObjProvider::a_child_of_b( $obj, $app ) &&
-            $app->_fetch_permitted( $obj, $data ) )
+            $app->_fetch_permitted( $obj, $id ) )
         {
             return { r => $app->_obj_to_response( $obj ) };
         }
     }
-    return { err => "Unable to fetch $data->{id}" };
+    return { err => "Unable to fetch $id" };
 } #_fetch
 
 
@@ -382,7 +408,7 @@ sub _multi_fetch {
             my $obj = Yote::ObjProvider::fetch( $id );
             if( $obj &&
                 Yote::ObjProvider::a_child_of_b( $obj, $app ) &&
-                $app->_fetch_permitted( $obj, $data ) )
+                $app->_fetch_permitted( $obj, $id ) )
             {
                 $ret{$id} = $app->_obj_to_response( $obj );
             }
@@ -458,7 +484,7 @@ sub _obj_to_response {
             return $use_id if $xform_out;
             $d = $to_convert->{DATA};
             no strict 'refs';
-            $m = [ grep { $_ !~ /^(_.*|[A-Z].*|set_.*|get_.*|is|clone|can|fetch_root|import|init|isa|new|absorb)$/ } keys %{"${ref}\::"} ];
+            $m = [ grep { $_ && $_ !~ /^(_.*|[A-Z].*|set_.*|get_.*|is|clone|can|import|init|isa|new)$/ } keys %{"${ref}\::"} ];
             use strict 'refs';
         }
         return { a => ref( $self ), c => $ref, id => $use_id, d => $d, 'm' => $m };
@@ -477,17 +503,36 @@ Yote::AppRoot - Application Server Base Objects
 
 =head1 SYNOPSIS
 
-This object is meant to be extended to provide Yote apps.
+Extend this class to make an application, and fill it with methods that you want for your application.
+
 
 =head1 DESCRIPTION
 
 Each Web Application has a single container object as the entry point to that object which is an instance of the Yote::AppRoot class. A Yote::AppRoot extends Yote::Obj and provides some class methods and the following stub methods.
 
+=head2 Client Methods
+
+Clients automatically call the following methods on an application or the Yote Root application /
+
+=over 4
+
+=item create_account
+
+=item login
+
+=item verify_token
+
+=item remove_account
+
+=item reset_password
+
+=back
+
 =head2 CLASS METHODS
 
 =over 4
 
-=item fetch_root - returns the master root object.
+=item _fetch_root - returns the master root object.
 
 The master root object contains all web application roots. It is an AppRoot object.
 
@@ -507,7 +552,7 @@ Returns the root object. This is always object 1 for the App Server.
 
 =over 4
 
-=item account_root( login ) - Returns an account object associated with a login object.
+=item _account_root( login ) - Returns an account object associated with a login object.
 
 =back
 
